use crate::{schema, UIDCError, user_management};
use microrm::cli::CLIObject;

#[derive(clap::Parser, Debug)]
pub struct CreateParameters {
    username: String,
}

#[derive(Debug, clap::Subcommand)]
pub enum UserCommands {
    UpdateAuth {
        username: String,
        #[clap(short, long, default_value_t = false)]
        password: bool,
        #[clap(short, long, default_value_t = false)]
        totp: bool,
    }
}

impl CLIObject for schema::User {
    type Error = UIDCError;
    type CreateParameters = CreateParameters;

    fn create_from_params(cp: &Self::CreateParameters) -> Result<Self, UIDCError> {
        Ok(Self {
            username: cp.username.clone(),
            auth: Default::default(),
            groups: Default::default(),
        })
    }

    type ExtraCommands = UserCommands;
    type ExtraCommandData = schema::Realm;
    fn run_extra_command(
        realm: &schema::Realm,
        extra: &Self::ExtraCommands,
        _query_ctx: impl microrm::prelude::Queryable<EntityOutput = Self>,
        _insert_ctx: &impl microrm::prelude::Insertable<Self>) -> Result<(), UIDCError> {

        match extra {
            UserCommands::UpdateAuth { username, password, totp } => {
                user_management::change_auth(realm, username, *password, *totp)?;
            }
        }

        Ok(())
    }

    fn shortname(&self) -> &str {
        &self.username
    }
}
